@@ -24,14 +24,15 @@ import cgi
 import mimetypes
 import os
 import re
-import urlparse
+import sys
+import traceback
 try:
     from urlparse import parse_qs
 except ImportError:
     from cgi import parse_qs
 
 __author__ = 'Daniel Lindsley'
-__version__ = ('0', '6', '1')
+__version__ = ('0', '6', '2')
 __license__ = 'BSD'
 
 
@@ -94,12 +95,20 @@ HTTP_MAPPINGS = {
 class RequestError(Exception):
     """A base exception for HTTP errors to inherit from."""
     status = 404
+    
+    def __init__(self, message, hide_traceback=False):
+        super(RequestError, self).__init__(message)
+        self.hide_traceback = hide_traceback
 
 class Forbidden(RequestError):
     status = 403
 
 class NotFound(RequestError):
     status = 404
+    
+    def __init__(self, message, hide_traceback=True):
+        super(NotFound, self).__init__(message)
+        self.hide_traceback = hide_traceback
 
 class AppError(RequestError):
     status = 500
@@ -148,13 +157,13 @@ class Request(object):
             self.POST = self.build_complex_dict()
         elif self.method == 'PUT':
             self.PUT = self.build_complex_dict()
-
-
+    
+    
     def build_get_dict(self):
         """Takes GET data and rips it apart into a dict."""
         raw_query_dict = parse_qs(self._environ['QUERY_STRING'], keep_blank_values=1)
         query_dict = {}
-    
+        
         for key, value in raw_query_dict.items():
             if len(value) <= 1:
                 query_dict[key] = value[0]
@@ -162,10 +171,10 @@ class Request(object):
                 # Since it's a list of multiple items, we must have seen more than
                 # one item of the same name come in. Store all of them.
                 query_dict[key] = value
-    
+        
         return query_dict
-
-
+    
+    
     def build_complex_dict(self):
         """Takes POST/PUT data and rips it apart into a dict."""
         raw_data = cgi.FieldStorage(fp=self._environ['wsgi.input'], environ=self._environ)
@@ -225,7 +234,15 @@ def handle_request(environ, start_response):
 
 def handle_error(exception, request):
     """If an exception is thrown, deal with it and present an error page."""
-    request._environ['wsgi.errors'].write("Exception occurred on '%s': %s\n" % (request._environ['PATH_INFO'], exception))
+    if not getattr(exception, 'hide_traceback', False):
+        (e_type, e_value, e_tb) = sys.exc_info()
+        message = "%s occurred on '%s': %s\nTraceback: %s" % (
+            exception.__class__,
+            request._environ['PATH_INFO'],
+            exception,
+            ''.join(traceback.format_exception(e_type, e_value, e_tb))
+        )
+        request._environ['wsgi.errors'].write(message)
     
     if isinstance(exception, RequestError):
         status = getattr(exception, 'status', 404)
@@ -267,20 +284,17 @@ def content_type(filename):
     """
     ct = 'text/plain'
     ct_guess = mimetypes.guess_type(filename)
-
+    
     if ct_guess[0] is not None:
         ct = ct_guess[0]
     
     return ct
 
 
-# Static file handler
-
-def static_file(request, filename, root=MEDIA_ROOT):
+def static_file(filename, root=MEDIA_ROOT):
     """
-    Basic handler for serving up static media files.
-    
-    Accepts an optional root (filepath string, defaults to MEDIA_ROOT) parameter.
+    Fetches a static file from the filesystem, relative to either the given
+    MEDIA_ROOT or from the provided root directory.
     """
     if filename is None:
         raise Forbidden("You must specify a file you'd like to access.")
@@ -302,6 +316,25 @@ def static_file(request, filename, root=MEDIA_ROOT):
     return open(desired_path, 'r').read()
 
 
+# Static file handler
+
+def serve_static_file(request, filename, root=MEDIA_ROOT, force_content_type=None):
+    """
+    Basic handler for serving up static media files.
+    
+    Accepts an optional ``root`` (filepath string, defaults to ``MEDIA_ROOT``) parameter.
+    Accepts an optional ``force_content_type`` (string, guesses if ``None``) parameter.
+    """
+    file_contents = static_file(filename, root)
+    
+    if force_content_type is None:
+        ct = content_type(filename)
+    else:
+        ct = force_content_type
+    
+    return Response(file_contents, content_type=ct)
+
+
 # Decorators
 
 def get(url):
@@ -459,7 +492,7 @@ def run_itty(server='wsgiref', host='localhost', port=8080, config=None):
     name from WSGI_ADAPTERS to use an alternate WSGI server.
     """
     if not server in WSGI_ADAPTERS:
-        raise RuntimeError("Server '%s' is not a valid server. Please choose a different server.")
+        raise RuntimeError("Server '%s' is not a valid server. Please choose a different server." % server)
     
     if config is not None:
         # We'll let ImportErrors bubble up.
@@ -475,4 +508,7 @@ def run_itty(server='wsgiref', host='localhost', port=8080, config=None):
         print 'Use Ctrl-C to quit.'
         print
     
-    WSGI_ADAPTERS[server](host, port)
+    try:
+        WSGI_ADAPTERS[server](host, port)
+    except KeyboardInterrupt:
+        print 'Shutting down. Have a nice day!'
